@@ -20,8 +20,8 @@ tar -xJvf $CODE_ARCHIVE
 mkdir -p $INSTALL_DIR
 cd $CODE_DIR
 ./configure --prefix=/usr
-make
-DESTDIR=$INSTALL_DIR make install
+make -j2
+DESTDIR=$INSTALL_DIR make install-strip
 
 cd ..
 echo "*****************************************"
