@@ -29,4 +29,8 @@
 // main.cpp
 extern int arg_debug;
 
+// config.cpp
+void config_read_screen_size(int *x, int *y);
+void config_write_screen_size(int x, int y);
+
 #endif
