@@ -36,18 +36,24 @@
 #include "../../firetools_config.h"
 #include "../../firetools_config_extras.h"
 #include "pid_thread.h"
+#include "fstats.h"
 extern bool data_ready;
 
 static QString getName(pid_t pid);
+static QString getProfile(pid_t pid);
 static bool userNamespace(pid_t pid);
 static int getX11Display(pid_t pid);
 
 
 StatsDialog::StatsDialog(): QDialog(), mode_(MODE_TOP), pid_(0), uid_(0), 
 	pid_initialized_(false), pid_seccomp_(false), pid_caps_(QString("")), pid_noroot_(false),
-	pid_cpu_cores_(QString("")), pid_protocol_(QString("")), pid_name_(QString("")), pid_x11_(0),
+	pid_cpu_cores_(QString("")), pid_protocol_(QString("")), pid_name_(QString("")),
+	profile_(QString("")), pid_x11_(0),
 	have_join_(true), caps_cnt_(64), graph_type_(GRAPH_4MIN), no_network_(false) {
 
+	// clean storage area
+	cleanStorage();
+
 	procView_ = new QTextBrowser;
 	procView_->setOpenLinks(false);
 	procView_->setOpenExternalLinks(false);
@@ -58,7 +64,12 @@ StatsDialog::StatsDialog(): QDialog(), mode_(MODE_TOP), pid_(0), uid_(0),
 	QGridLayout *layout = new QGridLayout;
 	layout->addWidget(procView_, 0, 0);
 	setLayout(layout);
-	resize(650, 650);
+	
+	// set screen size and title
+	int x;
+	int y;
+	config_read_screen_size(&x, &y);
+ 	resize(x, y);
 	setWindowTitle(tr("Firetools"));
 	
 	// detect if joining a sandbox is possible on this system
@@ -95,6 +106,17 @@ StatsDialog::StatsDialog(): QDialog(), mode_(MODE_TOP), pid_(0), uid_(0),
 	
 }
 
+StatsDialog::~StatsDialog() {
+	if (!isMaximized())
+		config_write_screen_size(width(), height());
+}
+
+void StatsDialog::cleanStorage() {
+	storage_dns_ = "";
+	storage_caps_ = "";
+	storage_seccomp_ = "";
+}
+
 QString StatsDialog::header() {
 	QString msg;
 	if (mode_ == MODE_TOP) {
@@ -182,6 +204,8 @@ void StatsDialog::updateTree() {
 		return;
 	}
 
+	if (arg_debug)
+		printf("reading process tree configuration\n");
 	QString msg = header();
 	msg += "<hr><table><tr><td width=\"5\"></td><td>";
 		
@@ -226,37 +250,44 @@ void StatsDialog::updateSeccomp() {
 		return;
 	}
 
-	QString msg = header();
-	msg += "<hr><table><tr><td width=\"5\"></td><td>";
-		
-	char *str = 0;
-	char *cmd;
-	if (asprintf(&cmd, "firejail --seccomp.print=%d", pid_) != -1) {
-		str = run_program(cmd);
-		char *ptr = str;
-		// htmlize!
-		while (*ptr != 0) {
-			if (*ptr == '\n') {
-				*ptr = '\0';
-				msg += QString(str) + "<br/>\n";
-				ptr++;
-				
-				while (*ptr == ' ') {
-					msg += "&nbsp;&nbsp;";
+	QString msg = storage_seccomp_;
+	if (msg.isEmpty()) {
+		if (arg_debug)
+			printf("reading seccomp configuration\n");
+		QString msg = header();
+		msg += "<hr><table><tr><td width=\"5\"></td><td>";
+			
+		char *str = 0;
+		char *cmd;
+		if (asprintf(&cmd, "firejail --seccomp.print=%d", pid_) != -1) {
+			str = run_program(cmd);
+			char *ptr = str;
+			// htmlize!
+			while (*ptr != 0) {
+				if (*ptr == '\n') {
+					*ptr = '\0';
+					msg += QString(str) + "<br/>\n";
 					ptr++;
-				}	
-				str = ptr;
-				continue;
+					
+					while (*ptr == ' ') {
+						msg += "&nbsp;&nbsp;";
+						ptr++;
+					}	
+					str = ptr;
+					continue;
+				}
+				ptr++;
 			}
-			ptr++;
-		}
-		free(cmd);
-	}		
-
-	msg += "</td></tr></table>";
-	procView_->setHtml(msg);
+			free(cmd);
+		}		
+	
+		msg += "</td></tr></table>";
+		procView_->setHtml(msg);
+		storage_seccomp_ = msg;
+	}
 }
 
+
 void StatsDialog::updateCaps() {
 	int cycle = Db::instance().getCycle();
 	assert(cycle < DbPid::MAXCYCLE);
@@ -270,41 +301,47 @@ void StatsDialog::updateCaps() {
 		return;
 	}
 
-	QString msg = header();
-	msg += "<hr><table><tr><td width=\"5\"></td><td>";
-		
-	char *str = 0;
-	char *cmd;
-	if (asprintf(&cmd, "firejail --caps.print=%d", pid_) != -1) {
-		str = run_program(cmd);
-		char *ptr = str;
-		// htmlize!
-		int cnt = 0;
-		while (*ptr != 0) {
-			if (*ptr == '\n') {
-				// print only caps supported by the current kernel
-				if (cnt >= caps_cnt_)
-					break;
-				cnt++;
-				
-				*ptr = '\0';
-				msg += QString(str) + "<br/>\n";
+	QString msg = storage_caps_;
+	if (msg.isEmpty()) {
+		if (arg_debug)
+			printf("reading caps configuration\n");
+		msg = header();
+		msg += "<hr><table><tr><td width=\"5\"></td><td>";
+			
+		char *str = 0;
+		char *cmd;
+		if (asprintf(&cmd, "firejail --caps.print=%d", pid_) != -1) {
+			str = run_program(cmd);
+			char *ptr = str;
+			// htmlize!
+			int cnt = 0;
+			while (*ptr != 0) {
+				if (*ptr == '\n') {
+					// print only caps supported by the current kernel
+					if (cnt >= caps_cnt_)
+						break;
+					cnt++;
+					
+					*ptr = '\0';
+					msg += QString(str) + "<br/>\n";
+					ptr++;
+					
+	//				while (*ptr == ' ') {
+	//					msg += "&nbsp;&nbsp;";
+	//					ptr++;
+	//				}	
+					str = ptr;
+					continue;
+				}
 				ptr++;
-				
-//				while (*ptr == ' ') {
-//					msg += "&nbsp;&nbsp;";
-//					ptr++;
-//				}	
-				str = ptr;
-				continue;
 			}
-			ptr++;
-		}
-		free(cmd);
-	}		
-
-	msg += "</pre></td></tr></table>";
-	procView_->setHtml(msg);
+			free(cmd);
+		}		
+	
+		msg += "</pre></td></tr></table>";
+		procView_->setHtml(msg);
+		storage_caps_ = msg;
+	}
 }
 
 void StatsDialog::updateDns() {
@@ -320,36 +357,43 @@ void StatsDialog::updateDns() {
 		return;
 	}
 
-	QString msg = header();
-	msg += "<hr><table><tr><td width=\"5\"></td><td>";
-		
-	char *str = 0;
-	char *cmd;
-	if (asprintf(&cmd, "firejail --dns.print=%d", pid_) != -1) {
-		str = run_program(cmd);
-		char *ptr = str;
-
-		// htmlize!
-		while (*ptr != 0) {
-			if (*ptr == '\n') {
-				*ptr = '\0';
-				msg += QString(str) + "<br/>\n";
-				ptr++;
-				
-				while (*ptr == ' ') {
-					msg += "&nbsp;&nbsp;";
+	QString msg = storage_dns_;
+	if (msg.isEmpty()) {
+		if (arg_debug)
+			printf("reading dns configuration\n");
+			
+		msg = header();
+		msg += "<hr><table><tr><td width=\"5\"></td><td>";
+			
+		char *str = 0;
+		char *cmd;
+		if (asprintf(&cmd, "firejail --dns.print=%d", pid_) != -1) {
+			str = run_program(cmd);
+			char *ptr = str;
+	
+			// htmlize!
+			while (*ptr != 0) {
+				if (*ptr == '\n') {
+					*ptr = '\0';
+					msg += QString(str) + "<br/>\n";
 					ptr++;
-				}	
-				str = ptr;
-				continue;
+					
+					while (*ptr == ' ') {
+						msg += "&nbsp;&nbsp;";
+						ptr++;
+					}	
+					str = ptr;
+					continue;
+				}
+				ptr++;
 			}
-			ptr++;
-		}
-	}		
-	free(cmd);
-
-	msg += "</td></tr></table>";
-	procView_->setHtml(msg);
+		}		
+		free(cmd);
+	
+		msg += "</td></tr></table>";
+		procView_->setHtml(msg);
+		storage_dns_ = msg;
+	}
 }
 
 void StatsDialog::kernelSecuritySettings() {
@@ -361,6 +405,7 @@ void StatsDialog::kernelSecuritySettings() {
 	pid_caps_ = QString("");
 	pid_cpu_cores_ = QString("");
 	pid_protocol_ = QString("");
+	pid_mem_deny_exec_ = QString("disabled");
 
 	// caps
 	char *cmd;
@@ -407,10 +452,21 @@ void StatsDialog::kernelSecuritySettings() {
 		return;
 	str = run_program(cmd);
 	if (str) {
-		pid_protocol_ = QString(str);
+		if (strncmp(str, "Cannot", 6) == 0)
+			pid_protocol_ = QString("disabled");
+		else
+			pid_protocol_ = QString(str);
 	}
 	free(cmd);
 
+	if (asprintf(&cmd, "firejail --ls=%d /run/firejail/mnt", pid_) == -1)
+		return;
+	str = run_program(cmd);
+	if (str) {
+		if (strstr(str, "seccomp.mdwx"))
+			pid_mem_deny_exec_ = "enabled";
+	}
+	free(cmd);
 }
 
 
@@ -459,6 +515,7 @@ void StatsDialog::updatePid() {
 		kernelSecuritySettings();
 		pid_noroot_ = userNamespace(pid_);
 		pid_name_ = getName(pid_);
+		profile_ = getProfile(pid_);
 		pid_x11_ = getX11Display(pid_);
 		pid_initialized_ = true;
 	}
@@ -475,6 +532,9 @@ void StatsDialog::updatePid() {
 	if (!pid_name_.isEmpty())
 		msg += "<tr><td width=\"5\"></td><td><b>Sandbox name:</b> " + pid_name_ + "</td></tr>";
 	msg += "<tr><td width=\"5\"></td><td><b>Command:</b> " + QString(cmd) + "</td></tr>";
+	if (!profile_.isEmpty())
+		msg += "<tr><td width=\"5\"></td><td><b>Profile:</b> " + profile_ + "</td></tr>";
+	// add 
 	msg += "</table><br/>";
 
 	msg += "<table>";
@@ -519,11 +579,18 @@ void StatsDialog::updatePid() {
 		msg += QString("<td><b>Protocols:</b> disabled</td>");
 	msg += "</td></tr>";
 
+	msg += "<tr><td></td>";
+
 	// X11 display
 	if (pid_x11_) {
-		msg += "<tr><td></td><td><b>X11 Dispaly:</b> " + QString::number(pid_x11_) + "</td></tr>";
+		msg += "<td><b>X11 Dispaly:</b> " + QString::number(pid_x11_) + "</td>";
 	}
-	
+	else
+		msg +="<td></td>";
+
+	// memory deny exec
+	msg += "<td><b>Memory deny exec:</b> " + pid_mem_deny_exec_ + "</td></tr>";
+
 	// graph type
 	msg += "<tr></tr>";
 	msg += "<tr><td></td>";
@@ -586,6 +653,7 @@ void StatsDialog::cycleReady() {
 }
 
 void StatsDialog::anchorClicked(const QUrl & link) {
+	cleanStorage(); // full storage cleanup on any click
 	QString linkstr = link.toString();
 	
 	if (linkstr == "top") {
@@ -757,8 +825,26 @@ static QString getName(pid_t pid) {
 		errExit("asprintf");
 	FILE *fp = fopen(fname, "r");
 	if (fp) {
-		char name[50];
-		if (fgets(name, 50, fp))
+		char name[250];
+		if (fgets(name, 250, fp))
+			retval = QString(name);
+		fclose(fp);
+	}
+	free(fname);
+	
+	return retval;
+}
+
+static QString getProfile(pid_t pid) {
+	QString retval("");
+
+	char *fname;
+	if (asprintf(&fname, "/run/firejail/profile/%d", (int) pid) == -1)
+		errExit("asprintf");
+	FILE *fp = fopen(fname, "r");
+	if (fp) {
+		char name[250];
+		if (fgets(name, 250, fp))
 			retval = QString(name);
 		fclose(fp);
 	}
