@@ -35,6 +35,7 @@ Q_OBJECT
 
 public:
 	StatsDialog();
+	~StatsDialog();
 
 public slots:
 	void cycleReady();
@@ -49,6 +50,7 @@ private:
 	void updateSeccomp();	
 	void updateDns();	
 	void updateCaps();	
+	void cleanStorage();
 
 private:
 	QTextBrowser *procView_;
@@ -71,6 +73,8 @@ private:
 	QString pid_cpu_cores_;
 	QString pid_protocol_;
 	QString pid_name_;
+	QString pid_mem_deny_exec_;
+	QString profile_;
 	int pid_x11_;
 	
 	bool have_join_;
@@ -79,6 +83,11 @@ private:
 	bool no_network_;
 
 	PidThread *thread_;
+	
+	// storage for various sandbox settings
+	QString storage_dns_;
+	QString storage_caps_;
+	QString storage_seccomp_;
 };
 
 
